/*
 * $Id: main.c 2013-09-16 PM 6:42:57  ssk  $
 *
 * This program is Main of Noritake itron GU256X64-901A 256x64 Dot Matrix VFD Tutorial.
 * Designed for The Noritake high speed 8 bit parallel interface based VFDs on Atmels AVR MCUs
 *
 * Copyright (C) 2013  ssk.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

/**
 * \mainpage
 * a VFD emits a very bright light with high contrast and can support display elements of various colours
 * This project contains the source for a Noritake itron GU256X64-901A 256x64 Dot Matrix VFD
 * build around an Atmel ATMEGA324P Microcontroller.
 * Parts of this code might also be used in other projects and with
 * other microcontrollers.
 *
 * The code is distributed under the terms of the GNU General Public License.
 * VFD : Vacuum fluorescent display
 *
 * \author ssk.
*/

#include <avr/io.h>
#include <math.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "../lorem_ipsum_text.h"    // Placeholder text (dummy text)
#include "../vfd.h"
#include "../fonts.h"
#include "../uart.h"
#include "../i2cmaster.h"
#include "../eeprom.h"
#include "../bitmaps.h"


#define DELAY 3000     // milliseconds
#define SHOW_SECOND 10 // second

// Constants in program space        0....+....1....+....
PROGMEM prog_char pstr_welcome1[] = "Noritake itron";                // Welcome message display
PROGMEM prog_char pstr_welcome2[] = "GU256X64-901A";                 // Welcome message display
PROGMEM prog_char pstr_version1[] = "Dot Matrix VFD";                // Version number as string
PROGMEM prog_char pstr_version2[] = "Vacuum fluorescent display";    // Version number as string

// Initialize all data structures and electrical interfaces & Welcome Message
static void init(void);
static void demo_welcome(void);
static void pixel_test(void);
static void demo_line_graphics(void);
static void demo_bitmap(void);
static void demo_font(void);
static void demo_end(void);
static void drawSpinner(uint8_t pos, uint16_t x, uint8_t y);
static void lissajous_figure( uint16_t duration );
static void timer0_init(void);
static void timer1_init(void);
static volatile uint8_t  ISR_1second;
static volatile uint32_t ISR_milisecond = 0UL;	// milisecond

int main (void)
{
	init();
	sei();
	
	while(1)
	{
   _delay_ms(1500);
   demo_welcome();
   pixel_test();
   demo_line_graphics();
   demo_bitmap();
   demo_font();
   demo_end();
   _delay_ms(1000);
  }
}

// Initialize all data structures and electrical interfaces & Welcome Message
static void init(void)
{
	timer1_init();
  i2c_init();
  uart_init(UART_0, 57600);
  vfd_init();

  vfd_clear(0x00);
  vfd_clear_text(' ');
}
/*
* Welcome Message
*/
static void demo_welcome(void)
{
  vfd_clear(0x00);
  vfd_rect_round(0,0,VFD_X_PIXEL-1,VFD_Y_PIXEL-1,VFD_DOT_SET,VFD_LINE_SOLID,3);

  vfd_bitmap(_252x32_noritake, 1, 2);

  vfd_selectfont(corsiva_18_x23, VFD_DOT_SET, FONT_ENGLISH,0);
  vfd_gotoxy(8, 38); vfd_puts_p(pstr_version2);

  vfd_selectfont(system_5_5x7, VFD_DOT_SET,FONT_ENGLISH,1);
  vfd_gotoxy(106, 34); vfd_puts_p((char *)pstr_welcome2);



  vfd_rect_fill(4,60,VFD_X_PIXEL-1-2,33,VFD_DOT_XOR);
   _delay_ms(2000);

  // invert screen
  vfd_rect_fill(0,0,VFD_X_PIXEL-1,VFD_Y_PIXEL-1,VFD_DOT_XOR);
  _delay_ms(3000);
}

/*
 *
 * Bitmap display demo
 *
 */
static void demo_bitmap(void)
{
  vfd_clear(0x00);
  vfd_bitmap(_27x32_android ,0,0);
  vfd_selectfont(corsiva_18_x23, VFD_DOT_SET, FONT_ENGLISH,2);
  vfd_gotoxy(38, 2); vfd_puts_P("Bitmaps");
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_bitmap(_252x32_noritake, 1, 15);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_bitmap(_16x16_browser,0,  18);
  _delay_ms(300);
  vfd_bitmap(_16x16_camera,     _vfd_coord.x+8, 34);
  _delay_ms(300);
  vfd_bitmap(_16x16_dictionary, _vfd_coord.x+8, 18);
  _delay_ms(300);
  vfd_bitmap(_16x16_memo,       _vfd_coord.x+8, 34);
  _delay_ms(200);
  vfd_bitmap(_16x16_volume,     _vfd_coord.x+8, 18);
  _delay_ms(200);
  vfd_bitmap(_16x16_satellite,  _vfd_coord.x+8, 34);
  _delay_ms(200);
  vfd_bitmap(_16x16_clock,      _vfd_coord.x+8, 18);
  _delay_ms(300);
  vfd_bitmap(_16x16_no,         _vfd_coord.x+8, 34);
  _delay_ms(200);
  vfd_bitmap(_16x16_exclamation,_vfd_coord.x+8, 18);
  _delay_ms(200);
  vfd_bitmap(_17x16_ohm        ,_vfd_coord.x+7, 34);
  _delay_ms(200);
  vfd_bitmap(_17x16_volt      , _vfd_coord.x+6, 18);
  _delay_ms(DELAY);  

  vfd_clear(0x00);
  vfd_bitmap(_196x32_manmool, 30, 17);
  _delay_ms(DELAY);
}

/*
 *
 * Line Graphics demo
 *
 */
static void demo_line_graphics(void)
{
  uint8_t i,x,y,z,style;

  vfd_clear(0x00);
  vfd_cube(0,0,22,19, 10,10,32,29,VFD_DOT_SET,VFD_LINE_SOLID);

  vfd_selectfont(arial_narrow_13_x16b, VFD_DOT_SET, FONT_ENGLISH,2);
  vfd_gotoxy(38, 2); vfd_puts_P("LINE Graphics");
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_selectfont(system_5_5x7, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(1,0);  vfd_puts_P("Solid and Dotted");

  // horizontal line
  for(i=7;i<VFD_Y_PIXEL-1;i+=12)
  {
    vfd_hline(0, i, VFD_X_PIXEL-1-1, VFD_DOT_SET, VFD_LINE_SOLID);
    vfd_hline(0, i+6, VFD_X_PIXEL-1-1, VFD_DOT_SET, VFD_LINE_DOTTED);
    _delay_ms(100);
  }

  // vertical line
  for(i=0;i<(VFD_X_PIXEL/17);i++)
  {
    vfd_vline( i*20,    7,VFD_Y_PIXEL-1, VFD_DOT_SET, VFD_LINE_SOLID);
    vfd_vline((i*20)+10,7,VFD_Y_PIXEL-1, VFD_DOT_SET, VFD_LINE_DOTTED);
    _delay_ms(80);
  }
  _delay_ms(DELAY);

  vfd_clear(0x00);
  style = VFD_LINE_SOLID;
  for(i=0;i<25;i+=5)
  {
    vfd_rect(i, i, VFD_X_CENTER-i-2, VFD_Y_PIXEL-1-i, VFD_DOT_SET, style);
    style = (style == VFD_LINE_SOLID ? VFD_LINE_DOTTED: VFD_LINE_SOLID);
    vfd_rect_round(VFD_X_CENTER+i+2, i, VFD_X_PIXEL-1-i, VFD_Y_PIXEL-1-i, VFD_DOT_SET, style,3);
      vfd_gotoxy(36,28); vfd_puts_P("Rectangle");
    _delay_ms(400);
  }
  _delay_ms(DELAY);

  // draw a diagonal line
  vfd_clear(0x00);  // clear display
    vfd_gotoxy(0,0); vfd_puts_P("Diagonal Line");
  for(i=8;i<VFD_Y_PIXEL-1;i+=3)
  {
    vfd_line(0,8,VFD_X_PIXEL-1,i,VFD_DOT_SET,VFD_LINE_SOLID);
    _delay_ms(70);
  }
  x=14;
  y=34;
  z=0;
  for(i=60;i--;)
  {
    vfd_rect_fill(x-8,y-8,y,y+8,VFD_DOT_CLR); // clear previous spinner position
    drawSpinner(z++, x,y);    // draw new spinner position
    _delay_ms(20);
  }

  // draw a circle
  vfd_clear(0x00);
  vfd_selectfont(corsiva_18_x23, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(1,0); vfd_puts_P("Circle");
  for(x=0;x<15;x++)
  {
    for(i=0;i<15;i+=3)
    {
      vfd_circle(VFD_X_CENTER+x+6,VFD_Y_CENTER+x, i*2, VFD_DOT_SET);
      _delay_ms(60);

      vfd_circle(VFD_X_CENTER+x+6,VFD_Y_CENTER+x, i*2 , VFD_DOT_CLR);
      _delay_ms(0);
    }
  }

  vfd_rect(VFD_X_CENTER-1, 0, VFD_X_PIXEL-1, VFD_Y_PIXEL-1, VFD_DOT_SET, style);
  vfd_hline(VFD_X_CENTER-1, VFD_Y_CENTER, VFD_X_PIXEL-1, VFD_DOT_SET, VFD_LINE_DOTTED);
  vfd_vline(VFD_X_CENTER+32, 0, VFD_Y_PIXEL-1, VFD_DOT_SET, VFD_LINE_DOTTED);
  for(i=0;i<12;i++)
  {
    vfd_circle(VFD_X_CENTER+32,VFD_Y_CENTER, i*3 , VFD_DOT_SET);
    _delay_ms(400);
  }
  _delay_ms(DELAY);
}

static void drawSpinner(uint8_t pos, uint16_t x, uint8_t y)
{

  // this draws an object that appears to spin
  switch(pos % 8)
  {
    case 0 : vfd_line( x, y-8, x, y+8,    VFD_DOT_SET,VFD_LINE_SOLID);    break;
    case 1 : vfd_line( x+3, y-7, x-3, y+7,VFD_DOT_SET,VFD_LINE_SOLID);    break;
    case 2 : vfd_line( x+6, y-6, x-6, y+6,VFD_DOT_SET,VFD_LINE_SOLID);    break;
    case 3 : vfd_line( x+7, y-3, x-7, y+3,VFD_DOT_SET,VFD_LINE_SOLID);    break;
    case 4 : vfd_line( x+8, y, x-8, y,    VFD_DOT_SET,VFD_LINE_SOLID);    break;
    case 5 : vfd_line( x+7, y+3, x-7, y-3,VFD_DOT_SET,VFD_LINE_SOLID);    break;
    case 6 : vfd_line( x+6, y+6, x-6, y-6,VFD_DOT_SET,VFD_LINE_SOLID);    break;
    case 7 : vfd_line( x+3, y+7, x-3, y-7,VFD_DOT_SET,VFD_LINE_SOLID);    break;
  }
}
/*
 *
 * Hangul, English, Number Font Demo
 *
 */
static void demo_font(void)
{
  vfd_clear(0x00);
  vfd_selectfont(TB24G_FNT_24X24, VFD_DOT_SET, FONT_HANGUL,0);
  vfd_gotoxy(0,8); vfd_puts_P("");
  vfd_selectfont(arial_narrow_31_x39b, VFD_DOT_XOR, FONT_ENGLISH,0);
  vfd_gotoxy(26,0); vfd_puts_P("A");
  vfd_selectfont(arial_narrow_31_x39b, VFD_DOT_SET, FONT_ENGLISH,0);
  vfd_gotoxy(54,0); vfd_puts_P("a");
  vfd_selectfont(TB24M_FNT_24X24, VFD_DOT_SET, FONT_HANGUL,0);
  vfd_selectfont(corsiva_18_x23, VFD_DOT_SET, FONT_ENGLISH,0);
  vfd_gotoxy(86,6); vfd_puts_P("۲ Fonts");
  _delay_ms(DELAY);	
	
  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(system_5_5x7, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P("SYSTEM_5_5X7");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(system_5_5x7			, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_p((char *)lorem_ipsum_text);
  _delay_ms(DELAY);
  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12	, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P("ARIAL_NARROW_8_X12");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(arial_narrow_8_x12		, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_p((char *)lorem_ipsum_text);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_9_x12b, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P("ARIAL_NARROW_9_X12B");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(arial_narrow_9_x12b		, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_p((char *)lorem_ipsum_text);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_12_x16	, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P("ARIAL_NARROW_12_X16");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(arial_narrow_12_x16		, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i=32;i<127;i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_13_x16b, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P("ARIAL_NARROW_13_X16B");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(arial_narrow_13_x16b		, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i=32;i<127;i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_18_x23b	, VFD_DOT_XOR, FONT_ENGLISH,1);
  vfd_puts_P(" ARIAL_NARROW_18_X23B ");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(arial_narrow_18_x23b			, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i=32;i<127;i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_18_x24, VFD_DOT_XOR, FONT_ENGLISH,1);
  vfd_puts_P(" ARIAL_NARROW_18_X24 ");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(arial_narrow_18_x24, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i=32;i<127;i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_24_x31b	, VFD_DOT_XOR, FONT_ENGLISH,1);
  vfd_puts_P(" ARIAL_24_X31B ");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(arial_narrow_24_x31b			, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i=32;i<127;i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_25_x32, VFD_DOT_XOR, FONT_ENGLISH,1);
  vfd_puts_P(" ARIAL_25_X32 ");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(arial_narrow_25_x32, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i=32;i<127;i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_31_x39b, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P("ARIAL_31_X39B");
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(corsiva_13_x16, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P("CORSIVA_13_x16");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(corsiva_13_x16			, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_p((char *)lorem_ipsum_text);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(corsiva_18_x23	, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P("CORSIVA_18_x23");

  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(corsiva_18_x23			, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i=32;i<127;i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P(" FIXEDNUMS_13_7X15 ");

  vfd_selectfont(fixednums_13_7x15, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i='*'; i<'<'+1; i++) vfd_putc(i);

  vfd_gotoxy(0,_vfd_coord.y + _vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i='*'; i<'<'+1; i++) vfd_putc(i);

  vfd_gotoxy(0,_vfd_coord.y + _vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i='*'; i<'<'+1; i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P(" FIXEDNUMS_21_12X24 ");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(fixednums_21_12x24, VFD_DOT_SET, FONT_ENGLISH,1);
  for(uint8_t i='*'; i<'<'+1; i++) vfd_putc(i);

  vfd_gotoxy(0,_vfd_coord.y + _vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i='*'; i<'<'+1; i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P(" FIXEDNUMS_27_16X32 ");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(fixednums_27_16x32, VFD_DOT_SET, FONT_ENGLISH,1);
  for(uint8_t i='*'; i<'8'+1; i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P(" FIXEDNUMS_34_20X39 ");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(fixednums_34_20x39, VFD_DOT_SET, FONT_ENGLISH,1);
  for(uint8_t i='0'; i<';'+1; i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P(" FIXEDNUMS_40_22X47 ");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(fixednums_40_22x47, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P("24:59:59");
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P(" DIGITALNUMS_14_10X16 ");

  vfd_selectfont(digitalnums_14_10x16, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i='*'; i<'<'+1; i++) vfd_putc(i);

  vfd_gotoxy(0,_vfd_coord.y + _vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i='*'; i<'<'+1; i++) vfd_putc(i);

  vfd_gotoxy(0,_vfd_coord.y + _vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i='*'; i<'<'+1; i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P(" DIGITALNUMS_22_16X24 ");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(digitalnums_22_16x24, VFD_DOT_SET, FONT_ENGLISH,1);
  for(uint8_t i='0'; i<'?'+1; i++) vfd_putc(i);

  vfd_gotoxy(0,_vfd_coord.y + _vfd_fontread(_vfd_font+FONT_HEIGHT));
  for(uint8_t i='0'; i<'?'+1; i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P(" DIGITALNUMS_29_21X32 ");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(digitalnums_29_21x32, VFD_DOT_SET, FONT_ENGLISH,1);
  for(uint8_t i='0'; i<':'+1; i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P(" DIGITALNUMS_37_26X40 ");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(digitalnums_37_26x40, VFD_DOT_SET, FONT_ENGLISH,1);
  for(uint8_t i='0'; i<'8'+1; i++) vfd_putc(i);
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_8_x12, VFD_DOT_SET, FONT_ENGLISH,1);
  vfd_puts_P(" DIGITALNUMS_43_31X48 ");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT));
  vfd_selectfont(digitalnums_43_31x48, VFD_DOT_SET, FONT_ENGLISH,0);
  vfd_puts_P("24:59:59");
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_12_x16, VFD_DOT_SET, FONT_ENGLISH,0);
  vfd_selectfont(TBG1_FNT_16X16, VFD_DOT_SET, FONT_HANGUL,1);
  vfd_puts_P("¹ѱ 1.5 ü 16X16");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT)-2);
  vfd_puts_P("ѹα ְȭ̴. ѹα ֱ ο ְ,  Ƿ κ ´.");
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(TBG2_FNT_16X16, VFD_DOT_SET, FONT_HANGUL,1);
  vfd_puts_P("¹ѱ 1.5 ü 16X16");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT)-2);
  vfd_puts_P("ѹα ְȭ̴. ѹα ֱ ο ְ,  Ƿ κ ´.");
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(arial_narrow_18_x24, VFD_DOT_SET, FONT_ENGLISH,0);
  vfd_selectfont(TB24G_FNT_24X24, VFD_DOT_SET, FONT_HANGUL,1);
  vfd_puts_P("¹ѱ 24x24  ü");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT)+10);
  vfd_puts_P("! ѹα, ! ");
  _delay_ms(DELAY);

  vfd_clear(0x00);
  vfd_gotoxy(0,0);
  vfd_selectfont(TB24M_FNT_24X24, VFD_DOT_SET, FONT_HANGUL,1);
  vfd_puts_P("¹ѱ 24x24 ü");
  vfd_gotoxy(0,_vfd_fontread(_vfd_font+FONT_HEIGHT)+10);
  vfd_puts_P("Ӹ, ");
  _delay_ms(DELAY);
}
/*
 *
 * Display Panel Pixel test
 *
 */
static void pixel_test(void)
{
  uint16_t x;
  uint8_t y,c;

  vfd_clear(0x00);
  vfd_gotoxy(0,0);

  c = VFD_DOT_SET;
  for(x=VFD_X_PIXEL-1;x--;)
  {
    c = !c;
    for(y=VFD_Y_PIXEL-1;y--;)
    {
      c = !c;
      vfd_dot(x,y,c);
    }
    _delay_ms(5);
  }
  _delay_ms(3000);
}

static void demo_end(void)
{
  vfd_clear(0x00);
  vfd_gotoxy(44,24);
  vfd_selectfont(corsiva_13_x16, VFD_DOT_SET, FONT_ENGLISH,0);
  vfd_puts_P(" Thank you for Watching!. ");
  lissajous_figure(4000);
  _delay_ms(DELAY);  
}



/*
 * scribble drawing routine adapted from TellyMate scribble Video sketch
 * http://www.batsocks.co.uk/downloads/tms_scribble_001.pde
 */
static uint16_t fn_x( float tick )
{
  return (uint16_t)((VFD_X_PIXEL)/2 + ((VFD_X_PIXEL)/2-1) * sin( tick * 1.8 ) * cos( tick * 3.2 )) ;
}

static uint8_t fn_y( float tick )
{
  return (uint8_t)((VFD_Y_PIXEL)/2 + ((VFD_Y_PIXEL)/2 -1) * cos( tick * 1.2 ) * sin( tick * 3.1 )) ;
}

static void lissajous_figure( uint16_t duration )
{
  const  float tick = 1/128.0;
  float g_head_pos = 0.0;

  timer0_init();

  for(uint32_t start = ISR_milisecond;  ISR_milisecond - start < duration; )
  {
    g_head_pos += tick ;

    float head = g_head_pos ;
    float tail = head - (256 * tick) ;

    // set the pixels at the 'head' of the line...
    uint16_t  x = fn_x( head ) ;
    uint8_t   y = fn_y( head ) ;
    vfd_dot( x , y , VFD_DOT_SET) ;

    // clear the pixel at the 'tail' of the line...
    x = fn_x( tail ) ;
    y = fn_y( tail ) ;
    vfd_dot( x , y , VFD_DOT_CLR) ;
  }
}

static void timer0_init(void)
{
	ISR_milisecond = 0;
	TCCR0B = _BV(CS02);   	// timer0 prescale (256)
	TCNT0  =  177;				  // 1milisecond overflow (256)
	TIMSK0 = _BV(TOIE0);		// timer0 interrupt enable
	sei();

	// 20000000Hz / 256 prescaler = 78125Hz
	// 1 / 78125Hz = TCNT0 Increase elaspse time is 0.0000128 Second
	// 0.001 Second / 0.0000128 Second = 78(78.125)  TCNT0 reach 1milisecond
	// TCNT0 Value is 255 - 78 = 177 (1milisend overflow value)

}
ISR(TIMER0_OVF_vect)
{
	TCNT0 = 177;	          // 8bit timer initialize (1mili second)
	ISR_milisecond++;	      // 1 mili second
}
static void timer1_init(void)
{
	TCCR1B = _BV(CS12) | _BV(CS10);
  TCNT1  =  0xB3B4;	      // 16bit timer initialize (1mili second)
	TIMSK1 = _BV(TOIE1);		// timer1 interrupt enable

	// 20000000Hz / 1024 prescaler = 19531.25Hz
	// 1 / 19531.25Hz = TCNT1 Increase elaspse time is 0.0000512 Second
	// 1 Second / 0.0000512 Second = 19,531(19,531.25)  TCNT1 reach 1 Second
	// TCNT1 Value is 65,536 - 19,531 = 46,005(0xB3B5) (1 Second overflow value)
}

ISR(TIMER1_OVF_vect)
{
	TCNT1 = 0xB3B4;
	ISR_1second = 1; 	 // The elapsed time is a 1 Second
}

/*
 * EOF
 */