/*
 * $Id: eeprom.c 2013-02-25 16:25:49  ssk  $
 *
 * Atmels AVR MCUs library support for external I2C Serial EEPROMs
 *
 * This Program is Designed for 512kb I2C Atmel(AT24C512), OnSemi(CAT24C512) EEPROMs
 *
 * Copyright (C) 2013  ssk.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/

/**
 * \Revision History
 *
 * Note. The Revision History showns the most amenddment first
 *
 * 2013-08-06 PM 7:45:50  Add 1Mbit(128KByte) EEPROM(STMicroelectronics M24M01-R)
 *                        uint16_t addr -> uint32_t addr, 
 *                        EEPROM_MEMORY_PAGE_BIT add (for 128KByte EEPROM), 
 *
 * 2013-02-25 PM 4:25:49  Initial creation
 *
 *
 * \ssk
*/
#include <util/delay.h>
#include "i2cmaster.h"
#include "eeprom.h"
#include "uart.h"

#define	hibyte(x) (uint8_t)(((x) & 0xffff) >> 8)
#define	lobyte(x) (uint8_t) ((x) & 0x00ff)
/**
 * Write Operations (Byte Write)
 *
 * In Byte Write mode the Master sends a START, followed
 * by Slave address, two byte address and data to be written
 *
 * @param device Device Address
 * @param addr   16bit  Address
 * @param data   data to be written
 *
 * @return       0:Success
 *
 * @note :
 */
uint8_t i2c_eeprom_write(uint8_t device, uint32_t addr, uint8_t data)
{
	uint8_t ret;

	ret = i2c_eeprom_page_address_bitset(device, addr);
  if(ret) 
 	{ 
 		i2c_stop();
 		
 		if(EEPROM_MESSAGE_UART_OUT)
    {
      uart_puts_P(UART_0, "[ERROR] i2c_eeprom_write() ->i2c_eeprom_page_address_bitset() fail[");
      uart_puts_ultoa(UART_0, device,2,8,'0');
      uart_puts_P(UART_0, "][");
      uart_puts_ultoa(UART_0, ret,10,2,'0');
      uart_puts_P(UART_0, "]\r\n");
    }
 		return ret;
 		
 	}

  ret = i2c_write(data);
  if(ret) 
 	{ 
 		i2c_stop();
  	return EEPROM_RET_WRITE_FAIL;
  }

  // terminate the write sequence
  i2c_stop();

  // EEPROM enters an internally-timed write cycle, tWR, to the nonvolatile memory
  /*
   * Maximum tWR(Write Cycle Time) : 5ms
   * STOP CONDIION <----- tWR -----> START CONDITION
   *               |----  5ms -----|
   */
  _delay_ms(5);

  return 0;
}
/**
 * Write Operations (Page Write)
 *
 * A Page Write is initiated the same way as a byte write, but the microcontroller
 * does not send a Stop condition after the first data word is clocked in
 *
 * @param device Device Address
 * @param addr   16bit Address
 * @param data   PROGMEM pointer to be written
 * @param size   data size(byte)
 *
 * @return       0:Success
 *
 * @note : EEPROM_PAGE_SIZE
 *    256, 1024Kbit(1Megabit) = 1,048,576bit = 131,072Byte(1,048,576/8bit) = 128Kbyte(131,072/1024Byte) 
 *    128,  512Kbit( 512Kbit) =   524,288bit =  65,536Byte(  524,288/8bit) =  64Kbyte( 65,536/1024Byte) 
 *     64,  256Kbit( 256Kbit) =   262,144bit =  32,768Byte(  262,144/8bit) =  32Kbyte( 32,768/1024Byte) 
 *
 */
uint8_t i2c_eeprom_write_page(uint8_t device, uint32_t addr, uint8_t *data, uint16_t size)
{
  uint32_t startaddr, endaddr;
  uint16_t calcpage,i;
  uint8_t over64K=0;     // 0: current addr is <= 0xffff(65535)
  uint8_t c,ret,pagecnt=0;

  uint16_t writecount=0; // UART for message ON

  startaddr = addr;
  endaddr   = addr + size;

  if(EEPROM_MESSAGE_UART_OUT)
  {
     uart_puts_P(UART_0, "[START Page Write] Dev#:");
     uart_puts_ultoa(UART_0, device,2,8,'0');
     uart_puts_P(UART_0, ", Page Sz:");
     uart_puts_ultoa(UART_0, EEPROM_PAGE_SIZE,10,3,'0');
     uart_puts_P(UART_0, ", Size:");
     uart_puts_ultoa(UART_0, size,10,5,'0');
     uart_puts_P(UART_0, ", Start:");
     uart_puts_ultoa(UART_0, startaddr,10,6,'0');
     uart_puts_P(UART_0, ", End:");
     uart_puts_ultoa(UART_0, endaddr-1,10,6,'0');
     uart_puts_P(UART_0, "\r\n");
  }

  while(startaddr < endaddr)
  {
    // Calculate the number of bytes from the current address to the end of the current page
    // IF, use different density EEPROM, you should MODIFY 'EEPROM_PAGE_SIZE'
    calcpage = (EEPROM_PAGE_SIZE - (startaddr % EEPROM_PAGE_SIZE));

    // If there are less bytes left than the length of the page , write only the bytes left
    if((startaddr + calcpage) >= endaddr) calcpage = endaddr - startaddr;

    if(startaddr > 0xffff ) over64K = 1;
    	
   	ret = i2c_eeprom_page_address_bitset(device, startaddr);
   	if(ret) 
   	{ 
   		i2c_stop();
   		
 		  if(EEPROM_MESSAGE_UART_OUT)
      {
        uart_puts_P(UART_0, "[ERROR] i2c_eeprom_write_page() ->i2c_eeprom_page_address_bitset() fail[");
        uart_puts_ultoa(UART_0, device,2,8,'0');
        uart_puts_P(UART_0, "][");
        uart_puts_ultoa(UART_0, ret,10,2,'0');
        uart_puts_P(UART_0, "]\r\n");
      }   		
   		return ret;
   	}
   	
    for( i = 0; i < calcpage; i++ )
    {
    	 if(((startaddr+i) > 0xffff ) && (over64K == 0))
    	 {
    	 	 over64K = 1;
    	 	 ret = i2c_eeprom_page_address_bitset(device, startaddr+i);
         { 
   		     i2c_stop();
 		       if(EEPROM_MESSAGE_UART_OUT)
           {
             uart_puts_P(UART_0, "[ERROR] i2c_eeprom_write_page() ->i2c_eeprom_page_address_bitset() fail[");
             uart_puts_ultoa(UART_0, device,2,8,'0');
             uart_puts_P(UART_0, "][");
             uart_puts_ultoa(UART_0, ret,10,2,'0');
             uart_puts_P(UART_0, "]\r\n");
           }     		     
   		     return ret;
         }
    	 }
    	 
       c = pgm_read_byte_near(data++);
       ret = i2c_write(c);
       if(ret) return EEPROM_RET_WRITE_FAIL;

       // UART for messgae(debugging, logging, etc.,)
       if(EEPROM_MESSAGE_UART_OUT)
       {
       	 uart_puts_ultoa(UART_0, device,2,8,'0');
         uart_puts_P(UART_0, ", Page:");
         uart_puts_ultoa(UART_0, pagecnt,10,2,'0');
         uart_puts_P(UART_0, ", Count:");
         uart_puts_ultoa(UART_0, EEPROM_PAGE_SIZE,10,3,'0');
         uart_puts_P(UART_0, "/");         
         uart_puts_ultoa(UART_0, i,10,3,'0');
         uart_puts_P(UART_0, ", ADDR:");
         uart_puts_ultoa(UART_0, endaddr-1,10,6,'0');
         uart_puts_P(UART_0, "/");
         uart_puts_ultoa(UART_0, addr+writecount,10,6,'0');
         uart_puts_P(UART_0, ", Total Write:");
         uart_puts_ultoa(UART_0, writecount,10,6,'0');
         uart_puts_P(UART_0, ", [");
         uart_puts_ultoa(UART_0, c,16,2,'0');
         uart_puts_P(UART_0, "]\r\n");
         writecount++;
       }
    }

    // terminate the write sequence
    i2c_stop();
    // EEPROM enters an internally-timed write cycle, tWR, to the nonvolatile memory
    /*
     * Maximum tWR(Write Cycle Time) : 5ms
     * STOP CONDIION <----- tWR -----> START CONDITION
     *               |----  5ms -----|
     */
    _delay_ms(5);
    startaddr+= calcpage;
    pagecnt++;
  }
  return 0;
}
/**
 * Byte Read  Operations(Selective Read)
 *
 * The Read operation can also be started at an address different
 * from the one stored in the internal address counter
 *
 * @param device Device Address
 * @param addr   32Bit Address
 * @param data   data to be written
 *
 * @return       read data
 *
 * @note :
 */
uint8_t i2c_eeprom_read(uint8_t device, uint32_t addr)
{
	uint8_t data,ret;

	ret = i2c_eeprom_page_address_bitset(device, addr);
  if(ret) 
  {
  	i2c_stop();
  	
  	if(EEPROM_MESSAGE_UART_OUT)
    {
      uart_puts_P(UART_0, "[ERROR] i2c_eeprom_read() ->i2c_eeprom_page_address_bitset() fail[");
      uart_puts_ultoa(UART_0, device,2,8,'0');
      uart_puts_P(UART_0, "][");
      uart_puts_ultoa(UART_0, ret,10,2,'0');
      uart_puts_P(UART_0, "]\r\n");
    }  
  	return 0;
  }

  ret = i2c_rep_start(device+I2C_READ);
  if(ret) 
  {
  	i2c_stop();
    if(EEPROM_MESSAGE_UART_OUT)
    {
      uart_puts_P(UART_0, "[ERROR] i2c_eeprom_read() ->i2c_rep_start() fail[");
      uart_puts_ultoa(UART_0, device,2,8,'0');
      uart_puts_P(UART_0, "][");
      uart_puts_ultoa(UART_0, ret,10,2,'0');
      uart_puts_P(UART_0, "]\r\n");
    }
  	return 0;
  }
  
  data = i2c_readNak();
  i2c_stop();

  return data;

}
/**
 * Byte Read Operations (Sequential Read)
 *
 * Sequential Reads are initiated by a Random Address Read
 *
 * @param device Device Address
 * @param addr   32Bit Address
 *
 * @return       0:Success
 *
 * @note : Sequential Read is 5 times Faster than Selective Read(Immediate Address Read)
 *
 */
uint8_t i2c_eeprom_read_seq_start(uint8_t device, uint32_t addr)
{
	uint8_t ret;

	ret = i2c_eeprom_page_address_bitset(device, addr);
  if(ret) 
 	{ 
 		i2c_stop();
 		return ret;
 	}

  ret = i2c_rep_start(device+I2C_READ);
  if(ret) 
 	{ 
 		i2c_stop();
  	return EEPROM_RET_READ_FAIL;
  }
  
  return 0;
}

uint8_t i2c_eeprom_read_seq(void)
{
  return i2c_readAck();

}
void i2c_eeprom_read_seq_stop(void)
{
	i2c_readNak();
  i2c_stop();
}

/**
 * A Memory page address bit Check (for 128KByte EEPROM)
 *
 * if address > 0xffff(65535) then A Memory page address bit SET (P0 = '1')
 *
 * @param device Device Address
 * @param addr   32Bit Address
 *
 * @return       0:Success
 *
 * @note : 
 *
 */
uint8_t i2c_eeprom_page_address_bitset(uint8_t device, uint32_t addr)
{
  if(addr > 0xffff)
  {
  	// UART for messgae(debugging, logging, etc.,)
    if(EEPROM_MESSAGE_UART_OUT)
    {
      uart_puts_P(UART_0, "A Memory page address bit SET! BEFORE Device[");
      uart_puts_ultoa(UART_0, device,2,8,'0');
      uart_puts_P(UART_0, "]  r/w [addr:");
      uart_puts_ultoa(UART_0, addr,16,5,'0');
      uart_puts_P(UART_0, "]\r\n");
    }
 
   	device |=  _BV(EEPROM_MEMORY_PAGE_BIT);
   	
   	// UART for messgae(debugging, logging, etc.,)
    if(EEPROM_MESSAGE_UART_OUT)
    {
      uart_puts_P(UART_0, "A Memory page address bit SET! AFTER  Device[");
      uart_puts_ultoa(UART_0, device,2,8,'0');
      uart_puts_P(UART_0, "]  r/w [addr:");
      uart_puts_ultoa(UART_0, addr,16,5,'0');
      uart_puts_P(UART_0, "]\r\n");
    }
  }	
  else
  {
   	device &= ~_BV(EEPROM_MEMORY_PAGE_BIT);
  }

  if(i2c_start(device+I2C_WRITE)) return EEPROM_RET_I2C_START_FAIL;
  
  if(i2c_write(hibyte(addr))) return EEPROM_RET_WRITE_AH_FAIL;
  if(i2c_write(lobyte(addr))) return EEPROM_RET_WRITE_AL_FAIL;
	
	return 0;
}


/*
 * EOF
 */