/*
* $Id: fonts.c 2013-02-20 13:51:44 ssk $
*
* Atmels AVR MCUs library support for graphic LCDs
*
* Supports fonts that are defined in bit map tables in allfonts.h
* The font tables in this file were derived from a conversion program called TDF(The Dot Factory)
* by Eran Duchan, Israel
*
* This Program is Designed for EPSON SED1335F0A
*
* Copyright (C) 2012  ssk.
*
* Originally Adapted from
*
*    Eran Duchan, Israel <http://www.pavius.net/2009/07/the-dot-factory-an-lcd-font-and-image-generator/>
*    Kaiserslautern, Germany <eversmith@heizung-thomas.de>
*    Martin Thomas, Kaiserslautern, Germany <eversmith@heizung-thomas.de>
*
* Changed & added additional constants for gcc version 4.3.3 (WinAVR 20100110), improved speed and optimized code
* You need to be selective about the information you receive instead of accepting it unconditionally.
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* Caveats: this VFD module has Pixel orientation per Byte: D7..D0 (MSB First) characteristics. 
*          refer to a _reverse_bits(uint8_t data);
*
*/

/**
* \Revision History
*
* Note. The Revision History showns the most amenddment first  
* 2013-10-06 AM 6:03:29
* 2013-10-03 PM 1:40:46  revoke font color parameter  void vfd_offsetwrite(uint8_t data);
* 2013-09-30 PM 4:01:03  add Font Color add (XOR,SET) void vfd_putc(uint8_t c); , void vfd_putcK(uint8_t c);
*                        add font color parameter     void vfd_offsetwrite(uint8_t data, uint8_t color);
*
* 2013-09-21 PM 4:02:21  Modify in order that using Noritake itron GU256X64-901A 256x64 Dot Matrix VFD
*                        vfd_offsetwrite(uint8_t data); - modified for Column major font write
* 2013-08-07 AM 2:31:07  add Hangul space between the lines
*                        #define KFONT_SPACE_BTL
* 2013-08-05 PM 2:08:17  improvement to allow for better readability
*                        Several function were consolidated into one(Hangul/English Font Function Pointer)
* 2013-08-05 AM 2:16:32  bug fix(cut character byte of line end)
*                        _glcd_offsetwrite(uint8_t data)
* 2013-08-02 AM 1:32:12  Add Hangul
*                        glcd_putcK(uint16_t kc)
* 2013-08-01 AM 1:08:06  bug fix(Word wrap check) & improvement to allow for better readability
*                        glcd_putc(uint8_t c)
* 2013-02-20 PM 13:51:44 Initial creation
*
* \ssk
*/
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "vfd.h"
#include "fonts.h"
#include "eeprom.h"
#include "allfonts.h"
#include "uart.h"


// Font Indices
#define FONT_SIZE       	       0     // byte
#define FONT_FIXED_WIDTH	       2     // pixel
#define FONT_HEIGHT		   	       3     // pixel
#define FONT_FIRST_CHAR		       4
#define FONT_END_CHAR 		       5
#define FONT_WIDTH_TABLE	       6     // bytes

#define KFONT_SPACE_BTL          0     // Hangul space between the lines
#define KFONT_HEADER_SZ          4     // Hangul font header size
#define KFONT_CNV_HEADER_SZ      2     // Hangul code converter index table header size


#define MAX(a,b) (((a)>(b))?(a):(b))

static void _vfd_puts(uint8_t c, uint8_t k);
static uint8_t _reverse_bits(uint8_t data);
static uint8_t _mask[] = {0xff,0x80,0xc0,0xe0,0xf0,0xf8,0xfc,0xfe};

uint8_t   _vfd_fontcolor;
 int8_t   _vfd_sbl;        // space between letters;

vfd_font_callback	_vfd_fontread;

#if(FONT_EXTERNAL_EEPROM)
  #warning "EXTERNAL EEPROM FONT Installed!!!."
  uint32_t  _vfd_font;
  uint32_t  _vfd_Kfont;
  uint8_t vfd_readfont(const uint32_t addr)
  {
  	return i2c_eeprom_read(EEPROM_I2C_DEVICE, addr);
  }
#else
  uint8_t*  _vfd_font;
  uint8_t*  _vfd_Kfont;
  uint8_t vfd_readfont(const uint8_t* ptr)
  {
  	return pgm_read_byte(ptr);
  }

#endif

#if(FONT_EXTERNAL_EEPROM)
  #warning "EXTERNAL EEPROM FONT Installed!!!."
  void vfd_selectfont(uint32_t font, uint8_t color, uint8_t type, int8_t sbl)
#else
  void vfd_selectfont(uint8_t* font, uint8_t color, uint8_t type, int8_t sbl)
#endif
  {
    #if(FONT_HANGUL)
      if(type == FONT_HANGUL)
       	_vfd_Kfont = font;        // save new Hangul font
      else
    #endif
     	  _vfd_font  = font;        // save new font

   	_vfd_fontcolor = color;       // save new font color
   	_vfd_fontread  = vfd_readfont;
   	_vfd_sbl       = sbl;         // save Space between letter
  }

/**
 * output a Hangul
 *
 * @param kc the character to output
 *
 * If the character will not fit on the current text line
 * inside the text area,
 * the text position is wrapped to the next line.
 *
 */
#if(FONT_HANGUL)
void vfd_putcK(uint16_t kc)
{
	// no font installed
	if((FONT_EXTERNAL_EEPROM == 0) && (_vfd_Kfont == 0)) return;

	uint8_t data;

  uint16_t kssm;
  uint16_t x,y,index;
  uint16_t chosung, joongsung, jongsung;

  uint8_t height    = _vfd_fontread(_vfd_font+FONT_HEIGHT);
  uint8_t kwidth    = _vfd_fontread(_vfd_Kfont+FONT_FIXED_WIDTH);
  uint8_t kheight   = _vfd_fontread(_vfd_Kfont+FONT_HEIGHT);
  uint8_t page      = (kheight+7)/8;

	// out out range
  if((_vfd_coord.y+8) > VFD_Y_PIXEL) return;

	// Word wrap check
	if((_vfd_coord.x + kwidth + _vfd_sbl) >  VFD_X_PIXEL)
  {
  	// i dont want Word wrap
    if(FONT_NO_WORD_WRAP) return;

  	vfd_gotoxy(0, _vfd_coord.y+height);
  }

  kssm = ks2kssm((uint8_t)((kc&0xff00) >> 8),(uint8_t)kc&0x00ff);

  #if(FONT_EXTERNAL_EEPROM)
    chosung   = i2c_eeprom_read(EEPROM_I2C_DEVICE, KFONT_CNV_HEADER_SZ+ks2kssm_index+kssm*3);           // ʼ
    joongsung = i2c_eeprom_read(EEPROM_I2C_DEVICE, KFONT_CNV_HEADER_SZ+ks2kssm_index+kssm*3+1)+161;     // ߼
    jongsung  = i2c_eeprom_read(EEPROM_I2C_DEVICE, KFONT_CNV_HEADER_SZ+ks2kssm_index+kssm*3+2);         // 
  #else
    chosung   = pgm_read_byte(KFONT_CNV_HEADER_SZ+ks2kssm_index+kssm*3);           // ʼ
    joongsung = pgm_read_byte(KFONT_CNV_HEADER_SZ+ks2kssm_index+kssm*3+1)+161;     // ߼
    jongsung  = pgm_read_byte(KFONT_CNV_HEADER_SZ+ks2kssm_index+kssm*3+2);         // 
  #endif

  if(jongsung) jongsung+=248;

  chosung   *= (kwidth * page);
  joongsung *= (kwidth * page);
  jongsung  *= (kwidth * page);

  chosung   += KFONT_HEADER_SZ;  // add offset (Hangule Font Header Size)
  joongsung += KFONT_HEADER_SZ;
  jongsung  += KFONT_HEADER_SZ;

  x = _vfd_coord.x;
  y = _vfd_coord.y;

  for(uint8_t p=0; p<page; p++) //  each vertical byte
  {
  	for(uint8_t kw=0; kw<kwidth; kw++) // each Horizontal pixel
  	{
  		index = (kwidth*p)+kw;

      data = _vfd_fontread(_vfd_Kfont+chosung  +index) |
             _vfd_fontread(_vfd_Kfont+joongsung+index) |
             _vfd_fontread(_vfd_Kfont+jongsung +index);

      data = _reverse_bits(data);

     	if(_vfd_fontcolor == VFD_DOT_XOR) data = ~data;

      vfd_gotoxy(x+kw,_vfd_coord.y);
      vfd_offsetwrite(data);

    }
    vfd_gotoxy(x, _vfd_coord.y+8);
  }
	vfd_gotoxy(x+kwidth+_vfd_sbl, y);
}
#endif
/**
 * output a character
 *
 * @param c the character to output
 *
 * If the character will not fit on the current text line
 * inside the text area,
 * the text position is wrapped to the next line.
 *
 */
void vfd_putc(uint8_t c)
{
	if((FONT_EXTERNAL_EEPROM == 0) && (_vfd_font == 0)) return; // no font installed

  if(c < 0x20)  return; // special character

	uint16_t i,index = 0;
	uint16_t x;       // current X coordinate 0..255
	uint8_t  y;       // current Y coordinate 0..63 (LOOK! datatype uint8_t)
	uint8_t height    = _vfd_fontread(_vfd_font+FONT_HEIGHT);
	uint8_t page      = (height+7)/8;
	uint8_t firstChar = _vfd_fontread(_vfd_font+FONT_FIRST_CHAR);
	uint8_t charCount = _vfd_fontread(_vfd_font+FONT_END_CHAR) - _vfd_fontread(_vfd_font+FONT_FIRST_CHAR) + 1;

  uint8_t width;    // font width pixels
  uint8_t data,j;

  if((_vfd_coord.y) > VFD_Y_PIXEL-1) return;              // out out range
	if(c < firstChar || c >= (firstChar+charCount)) return; // invalid char

  c-= firstChar;

  if(isfixedwidth(_vfd_font))
  {
		width = _vfd_fontread(_vfd_font+FONT_FIXED_WIDTH);
		index = c*page*width+FONT_WIDTH_TABLE;
  }
  else
  {
  	// variable width font, read width data, to get the index
  	/*
  	 * Because there is no table for the offset of where the data
  	 * for each character glyph starts, run the table and add up all the
  	 * widths of all the characters prior to the character we need to locate.
  	 */

  	// read width data, to get the index
    for(i=0; i<c; i++) index += _vfd_fontread(_vfd_font+FONT_WIDTH_TABLE+i);

    index = index*page+charCount+FONT_WIDTH_TABLE;

  	// Finally, fetch the width of our character
  	width = _vfd_fontread(_vfd_font+FONT_WIDTH_TABLE+c);

  }

  // Word wrap check
	if((_vfd_coord.x + width + _vfd_sbl) >  VFD_X_PIXEL-1)
  {
  	// i dont want Word wrap
  	if(FONT_NO_WORD_WRAP) return;
  	
    vfd_gotoxy(0, _vfd_coord.y+height);
  }

  x = _vfd_coord.x;
  y = _vfd_coord.y;
  
  for( i=0; i<page; i++) /* each vertical byte */
 	{
 	  for( j=0; j<width; j++)   /* each Horizontal pixel */
    {
    	data = _reverse_bits(_vfd_fontread(_vfd_font+index+(i*width)+j));

      // add Font Color property
    	if((page > 0) && (i == page-1))
      {
        if(_vfd_fontcolor == VFD_DOT_XOR) data |= ~(_mask[height%8]);
      }

      vfd_gotoxy(x+j, _vfd_coord.y);
      if(_vfd_fontcolor == VFD_DOT_XOR) data = ~data;
      	
      vfd_offsetwrite(data);                         
  	}
  	
    // add sbl(space between letters)
    if(_vfd_coord.y < VFD_Y_PIXEL -1)
    {
      for( uint8_t sbl=1; sbl < _vfd_sbl+1; sbl++)
      { 
      	data = 0x00;
        // add Font Color property
        if((page > 0) && (i == page-1))
        {
          if(_vfd_fontcolor == VFD_DOT_XOR) data |= ~(_mask[height%8]);
        }
        
 	      if(_vfd_fontcolor == VFD_DOT_XOR) data = ~data;
 	      	
   	  	vfd_offsetwrite(data);
      }
    }
    
    if(_vfd_coord.y+8 > VFD_Y_PIXEL -1) break;

  	vfd_gotoxy(x, _vfd_coord.y+8);
	}
	vfd_gotoxy(x+width+_vfd_sbl, y);
}


// Character data put string
void vfd_puts(const char *str)
//void vfd_puts(const unsigned char *str)
{
	uint8_t c,k=0; // uknown character

  while((c = *str++))
  {
  	if(c&0x80) k = *str++;  // is Hangul ? then once more, Why? Hangul is 2Byte Chararter

   _vfd_puts(c, k );
  }
}
// PROGMEM data put string
void vfd_puts_p(PGM_P str)
{
	uint8_t c,k=0; // uknown character

  while((c = pgm_read_byte(str++)))
  {
    if(c&0x80) k = pgm_read_byte(str++); // is Hangul ? then once more, Why? Hangul is 2Byte Chararter

 	 _vfd_puts(c, k );
  }
}

static void _vfd_puts(uint8_t c, uint8_t k)
{
  uint16_t x     = _vfd_coord.x; // current X coordinate
  uint8_t height = _vfd_fontread(_vfd_font+FONT_HEIGHT);    // current english font height
    
  if(c&0x80) // Hangul
  {
    #if(FONT_HANGUL)
  	  uint16_t kc  = (((uint16_t)c)<<8) + (0xff&(k));
     	vfd_putcK(kc);
    #else
     	vfd_putc('?');
     	vfd_putc('?');
    #endif
  }
  else if(c == '\n') vfd_gotoxy(x, _vfd_coord.y + height);
  else vfd_putc(c); // English
}

/*
 * unsigned int to characters in LCD
 *
 * @param value       32bit unsigned int number to be displayed
 * @param radix         2(binary),
 *                     10(decimal),
 *                     16(hexadecimal)
 * @param length       display length (right alignment)
 * @param leadingchar  specify leading char
 *                     '0'  ->  '00099'
 *                     ' '  ->  '   99'
 *                     '$'  ->  '$$$99'
 *                     'x'  ->  'xxx99'
 *
 */
#define NUMBER_OF_DIGITS 32
void vfd_puts_ultoa(uint32_t value, uint8_t radix, uint8_t length, uint8_t leadingchar)
{
  static uint8_t  index = NUMBER_OF_DIGITS;
  static unsigned char buffer[NUMBER_OF_DIGITS];

  do
  {
    buffer[--index] = '0' + (value % radix);
    if ( buffer[index] > '9') buffer[index] += 'A' - ':';  // continue with A, B,...

    value /= radix;
  } while (value != 0);

  while( index > (NUMBER_OF_DIGITS - length))
  {
  	buffer[--index] = leadingchar;
  }

  if(radix==16) vfd_puts_P("0x");
  else
  if(radix==2) vfd_puts_P("b");

  do
  {
  	vfd_putc(buffer[index++]);
  } while ( index < NUMBER_OF_DIGITS );
}

void vfd_puts_ltoa(int32_t value, uint8_t radix, uint8_t length, uint8_t leadingchar)
{
  if (value < 0 && radix == 10)
  {
    vfd_putc('-');
    vfd_puts_ultoa(-value, radix, length, leadingchar);
  }
  else
    vfd_puts_ultoa(value, radix, length, leadingchar);
}

/*
 * Reverse Bits of a byte
 *
 * Why reverse bits?
 *
 * I do not create work, use existing Column Major Font Data.
 * SAMSUNG KS0108 is a representative Controller of the Column Major
 * And Pixel orientation per Byte: D0..D7 (LSB First) characteristics.
 * LSB First rules generally apply to Column Major Coltroller
 * But Noritake VFD is D7..D0(MSB First) that is the reverse (Futaba Dot Matrix VFD are the same)
 *
 * That's the why.
 *
 * Column Major Display Pixel orientation per Byte (from top to bottom)
 *
 *      row             most cases   Noritake,Futaba
 *       0                 0               7
 *       1                 1               6
 *       2                 2               5
 *       3                 3               4
 *       4                 4               3
 *       5                 5               2
 *       6                 6               1
 *       7                 7               0
 *       8                 0               7
 *       9                 1               6
 *      10                 2               5
 *       .                 .               .
 *       .                 .               .
 *       .                 .               .
 *      64                 7               0
 *
 * Time Complexity: O(log n)
 *
 * O(n log n) is better performace but i cannot found googgle
 * http://www-graphics.stanford.edu/~seander/bithacks.html#BitReverseTable
 *
 */
static uint8_t _reverse_bits(uint8_t data)
{
  //uint8_t count   = sizeof(data) * 8 - 1;
  uint8_t count   = 7;
  uint8_t reverse = data;

  data >>= 1;
  while(data)
  {
    reverse <<= 1;
    reverse |= data & 1;
    data >>= 1;
    count--;
  }
  reverse <<= count;
  return reverse;
}

/*
 * EOF
 */
