/*
 * $Id: uart.c 2012-10-24 6:00  ssk  $
 *
 * This program is part of DALLAS DS1721 Temperature Sensor Tutorial.
 *
 * Copyright (C) 2012  ssk.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <avr/pgmspace.h>
#include "uart.h"

static int freeRam(void); 

void uart_init(uint8_t channel, uint16_t baudrate)
{
  uint16_t ubrr = ((F_CPU/(16UL*baudrate))-1);
  
  if(channel > MAX_UART_CHANNEL) channel = 0;  // Default UART Channel
  	
  if(channel == UART_0)
  {
    UBRR0H = (uint8_t)(ubrr>>8); 
    UBRR0L = (uint8_t)ubrr;      
    UCSR0B = (_BV(TXEN0) | _BV(RXEN0));
  }
  else
  {
    UBRR1H = (uint8_t)(ubrr>>8); 
    UBRR1L = (uint8_t)ubrr;      
    UCSR1B = (_BV(TXEN1) | _BV(RXEN1));
  }
  
  uart_puts_P(channel, "\r\n>>> Welcome Debug Mode ");
  uart_puts_P(channel, "UART:");
  uart_puts_ultoa(channel, channel,10,1,'0');
  uart_puts_P(channel, ", ");
  uart_puts_ultoa(channel, baudrate,10,6,' ');
  uart_puts_P(channel, "bps, UBRR:");
  uart_puts_ultoa(channel,ubrr,10,1,' ');
  uart_puts_P(channel,", Free memory:");
  uart_puts_ultoa(channel,freeRam(),10,4,' ');
  uart_puts_P(channel,"Byte\r\n");
}

void uart_putc(uint8_t channel, uint8_t data)
{
	if(channel == UART_0)
	{
  	loop_until_bit_is_set(UCSR0A, UDRE0);
    UDR0 = data;
  }
  else
	{
  	loop_until_bit_is_set(UCSR1A, UDRE1);
    UDR1 = data;
  }
}

void uart_puts(uint8_t channel, const char *str)
{
  register char c;
  while((c=*str++)) uart_putc(channel, c);

}

uint8_t uart_getc(uint8_t channel)
{
  if(channel == UART_0)
	{    
		loop_until_bit_is_set(UCSR0A, RXC0);
    return UDR0;
  }
  else
	{    
		loop_until_bit_is_set(UCSR1A, RXC1);
    return UDR1;
  }
  	
}

void uart_puts_p(uint8_t channel, const char *progmem_s)
{
  register char c;
  while((c= pgm_read_byte(progmem_s++))) uart_putc(channel, c);
}

/*
 * unsigned int to characters in UART
 *
 * @param value       32bit unsigned int number to be displayed
 * @param radix         2(binary),
 *                     10(decimal),
 *                     16(hexadecimal)
 * @param length       display length (right alignment)
 * @param leadingchar  specify leading char
 *                     '0'  ->  '00099'
 *                     ' '  ->  '   99'
 *                     '$'  ->  '$$$99'
 *                     'x'  ->  'xxx99'
 *
 */
#define NUMBER_OF_DIGITS 32
void uart_puts_ultoa(uint8_t channel, uint32_t value, uint8_t radix, uint8_t length, uint8_t leadingchar)
{
  static uint8_t  index = NUMBER_OF_DIGITS;
  static unsigned char buffer[NUMBER_OF_DIGITS];

  do
  {
    buffer[--index] = '0' + (value % radix);
    if ( buffer[index] > '9') buffer[index] += 'A' - ':';  // continue with A, B,...

    value /= radix;
  } while (value != 0);

  while( index > (NUMBER_OF_DIGITS - length))
  {
  	buffer[--index] = leadingchar;
  }

//  if(radix==16) uart_puts_P(channel, "0x");
//  else
//  if(radix==2) uart_putc('b');

  do
  {
  	uart_putc(channel, buffer[index++]);
  } while ( index < NUMBER_OF_DIGITS );
}
/*******************************************************************************
convert signed long int to characters in LCD
Input:    value unsigned long int number to be displayed
          radix  2(binary),
                10(decimal),
                16(hexadecimal)
Returns:  none
*******************************************************************************/
void uart_puts_ltoa(uint8_t channel, int32_t value, uint8_t radix, uint8_t length, uint8_t leadingchar)
{
  if (value < 0 && radix == 10)
  {
    uart_putc(channel, '-');
    uart_puts_ultoa(channel, -value, radix, length, leadingchar);
  }
  else
    uart_puts_ultoa(channel, value, radix, length, leadingchar);
}

/*
 * Determines how much RAM is currently unused
 *
 * http://jeelabs.org/2011/05/22/atmega-memory-use/
 * http://jeelabs.org/2011/12/05/types-of-memory-for-an-atmega/
 *
 */
static int freeRam(void) 
{
  extern int __heap_start, *__brkval; 
  
  int v;
  int a = (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
   
  return a;
}
/*
 * EOF
 */
