/*
* $Id: vfd.c 2013-09-21 PM 4:02:21 ssk  $
*
* This program is part of Noritake itron GU256X64-901A 256x64 Dot Matrix VFD Tutorial.
* Designed for The Noritake high speed 8 bit parallel interface based VFDs on Atmels AVR MCUs
*
* Copyright (C) 2013  ssk.
*
* Originally Adapted from
*    Most Functions   : GLCD Version 3 (Beta) Jun 9 2010 <http://arduino.cc/forum/index.php/topic,22208.0.html>
*    Font Function&Data: Martin Thomas, Kaiserslautern, Germany <eversmith@heizung-thomas.de>
*                       avr-gcc 3.4.3/avr-libc 1.2.3 (WinAVR 2/2005)
*    Cube Drawing     : Osama's Lab KS0108 GLCD library <https://sites.google.com/site/osamaslab>
*    Line Drawing     : http://en.wikipedia.org/wiki/Bresenham's_line_algorithm
*    Circle Drawing   : http://en.wikipedia.org/wiki/Midpoint_circle_algorithm
*
*
* Changed & added additional constants for gcc version 4.3.3 (WinAVR 20100110), improved speed and optimized code
* You need to be selective about the information you receive instead of accepting it unconditionally.
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*
*/

/*
*
* A Noritake itron GU256X64-901A 256x64 Dot Matrix Graphic/Text VFD module has been used
* for testing. This module not supports Busy Signal
*
* To the Memory of  Dr. Tadashi Nakamura
* in 1967, He is the inventor of a Vacuum fluorescent display called the VFD
* has passed away on March 25, 2013.
*
*/

/*
* \Revision History
*
* Note. The Revision History showns the most amenddment first
  2013-10-09 AM 5:50:39
  2013-10-06 AM 5:59:00   
*
* 2013-09-21 PM 4:02:21   Initial creation
*
* \ssk
*/

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "vfd.h"
#include "uart.h"

coordinates _vfd_coord;

static const uint8_t _mask_array[8] = {0x00,0x01,0x03,0x07,0x0f,0x1f,0x3f,0x7f};

static void _vfd_set_cursor_address(uint16_t addr);
static void _vfd_write_toggle(void);

// Display initalization
void vfd_init(void)
{
  // At Power on Rush, 1Sec Delay, if possible
  _delay_ms(1000);

  // Initialize VFD global variable (current x,y coordinate on display device)
  _vfd_coord.x = 0;
  _vfd_coord.y = 0;

  VFD_DATA_DDR  = 0xff;  // all output
  VFD_CTRL_DDR  |=  _BV(VFD_WR_PIN) | _BV(VFD_RD_PIN) | _BV(VFD_CD_PIN);
  VFD_CTRL_PORT |=  _BV(VFD_WR_PIN) | _BV(VFD_RD_PIN) | _BV(VFD_CD_PIN);

  // Initialize Display SRAM
  _vfd_set_cursor_address(0);  // Set Start Address 0
  vfd_command(VFD_DATA_WRITE);
  for(uint16_t i=VFD_SRAM_SZ;i--;) vfd_data(0x00);

  // Screen#1 (Graphic) Home Address
  vfd_command(VFD_START_ADDR_L1);
  vfd_data(lobyte(VFD_GRAPHIC_HOME));
  vfd_command(VFD_START_ADDR_U1);
  vfd_data(hibyte(VFD_GRAPHIC_HOME));

  // Screen#2 (Text) Home Address
  vfd_command(VFD_START_ADDR_L2);
  vfd_data(lobyte(VFD_TEXT_HOME));
  vfd_command(VFD_START_ADDR_U2);
  vfd_data(hibyte(VFD_TEXT_HOME));

  if(VFD_ADDR_AUTO_INC)
    vfd_command(VFD_ADDR_INC_AUTO);   // use Automatic Increment of Cursor address
  else
    vfd_command(VFD_ADDR_INC_HOLD);   // do not use Automatic Increment of Cursor address

//  vfd_command(VFD_DISPLAY_ON);    // omit this command, because Value is Power On. Defualt
//  vfd_command(VFD_SCREEN_OR);     // omit this command, because Value is Power On. Defualt
//  vfd_command(VFD_BRIGHTNESS_62); // omit this command, because 100%  is Power On. Defualt

  vfd_command(VFD_SCREEN_ON);     // Critical item(At Power on, Screen#1,#2 are Set to OFF)
}

/**
 * Send VFD controller instruction command
 * Input: instruction
 * Returns: none
*/
void vfd_command(const uint8_t command)
{
  VFD_CTRL_PORT |=   _BV(VFD_RD_PIN);
  VFD_CTRL_PORT |=   _BV(VFD_CD_PIN);

  VFD_DATA_PORT = command;

  _vfd_write_toggle();
}
/**
 * Send data byte to VFD controller
 * Input: data
 * Returns: none
*/
void vfd_data( uint8_t data)
{
  VFD_CTRL_PORT |=   _BV(VFD_RD_PIN);
  VFD_CTRL_PORT &= ~(_BV(VFD_CD_PIN));

  VFD_DATA_PORT  =  data;

  _vfd_write_toggle();
}
/*
 * Read a data byte from the given position
 *
 * @return Data read from VFD ram
 *
*/
uint8_t vfd_read_data(void)
{
  uint8_t data;

  VFD_DATA_DDR = 0x00;                    // all intput

  VFD_CTRL_PORT |=   _BV(VFD_WR_PIN);
  VFD_CTRL_PORT &= ~(_BV(VFD_CD_PIN));
  VFD_CTRL_PORT &= ~(_BV(VFD_RD_PIN));

  // Avoid Stupid Dummy Read
  // Read Signal Toggle (functions like a Dummy read but Faster than Dummy Read)
  VFD_CTRL_PORT |=   _BV(VFD_RD_PIN);
  VFD_CTRL_PORT &= ~(_BV(VFD_RD_PIN));

  // Since data excution time of the module is 2uSec
  // There is not provided Busy signal therefore more than 2uSec.
  // Refer. Datasheet 13. Interfacing, 14. Timing Chart
  _delay_us(2);

  data = VFD_DATA_INPUT;                  // Get data from VFD

  VFD_CTRL_PORT |=    _BV(VFD_RD_PIN);

  VFD_DATA_DDR = 0xff;                    // all output

  //vfd_gotoxy(_vfd_coord.x, _vfd_coord.y); // back to the original location.
  return(data);
}

/*
 * Send data byte to VFD controller (Pixel Aligment Write)
 * Input: data
 * Returns: none
 *
*/
void vfd_offsetwrite(uint8_t data)
{
  uint8_t y;
  uint8_t yoffset  = _vfd_coord.y % 8;
  uint8_t savedata;

  if(yoffset)
  {
    // first page
    vfd_command(VFD_DATA_READ);
    savedata =  vfd_read_data();
    
    
    savedata &= ~(_mask_array[8-yoffset]);  // Clear only display area
    savedata |= (data >> yoffset);
    
    vfd_command(VFD_DATA_WRITE);
    vfd_data(savedata);

    // second page
    y=8;
    if((_vfd_coord.y+8) > VFD_Y_PIXEL-1)
    {
      yoffset-= ((_vfd_coord.y+8) - VFD_Y_PIXEL);
      y = 0;
    }
    vfd_gotoxy(_vfd_coord.x, _vfd_coord.y+y);
    
    vfd_command(VFD_DATA_READ);
    savedata = vfd_read_data();
    
    savedata &= ~(_mask_array[yoffset] <<(8-yoffset)); // Clear only display area
    savedata |= data << (8-yoffset);

    vfd_command(VFD_DATA_WRITE);
    vfd_data(savedata);

    vfd_gotoxy(_vfd_coord.x+1, _vfd_coord.y-y);
  } 
  else
  {
    vfd_command(VFD_DATA_WRITE);
    vfd_data(data);

    vfd_gotoxy(_vfd_coord.x+1, _vfd_coord.y);
  }
}

// clear screen (graphic, Screen#1)
void vfd_clear(uint8_t fillchar)
{
  uint16_t i;
  // Automatic Increment of Cursor address  temporarily On.
  if(VFD_ADDR_AUTO_INC==0) vfd_command(VFD_ADDR_INC_AUTO);

  _vfd_set_cursor_address(VFD_GRAPHIC_HOME);

  vfd_command(VFD_DATA_WRITE);
  for(i=VFD_GRAPHIC_SIZE; i--;) vfd_data(fillchar);

  if(VFD_ADDR_AUTO_INC==0) vfd_command(VFD_ADDR_INC_HOLD);   
}



// clear screen (text, Screen#2)
void vfd_clear_text(uint8_t fillchar)
{
  uint8_t y,x;
  
  // Automatic Increment of Cursor address  temporarily On.
  if(VFD_ADDR_AUTO_INC==0) vfd_command(VFD_ADDR_INC_AUTO);

  _vfd_set_cursor_address(VFD_TEXT_HOME);

  for(y=VFD_Y_TEXT; y--;)
  {
    vfd_gotoxy_text(0,y);

    vfd_command(VFD_DATA_WRITE);
    for(x=VFD_X_TEXT; x--;) vfd_data(fillchar);
  }
  
  if(VFD_ADDR_AUTO_INC==0) vfd_command(VFD_ADDR_INC_HOLD);   
}
 /**
 * go to at x,y to the given position (graphic layer)
 *
 * @param x X coordinate, a value from 0 to VFD_X_PIXEL-1
 * @param y Y coordinate, a value from 0 to VFD_Y_PIXEL-1
 *
 * Sets the current position at location the specified x,y
 * x and y are relative to the 0,0 origin of the display which
 * is the upper left corner.
 * Requests to set pixels outside the range of the display will be ignored.
 *
 * @note
 *
*/
void vfd_gotoxy(uint16_t x,  uint8_t y)
{
  uint16_t addr;

  // exit if coordinates are not legal
  if( (x > VFD_X_PIXEL-1) || (y > VFD_Y_PIXEL-1) )  return;

  // save new coordinates
  _vfd_coord.x = x;
  _vfd_coord.y = y;

  y >>= 3;     // y=y/8, page coordinate of the cursor (0 ~ VFD_Y_PAGES-1)

  addr = VFD_GRAPHIC_HOME + (8 * x) + y;

  _vfd_set_cursor_address(addr);
}
 /**
 * go to at x,y to the given position (text layer)
 *
 * @param x X coordinate, a value from 0 to  VFD_X_TEXT-1
 * @param y Y coordinate, a value from 0 to  VFD_Y_TEXT-1
 *
 * Sets the current position at location the specified x,y
 * x and y are relative to the 0,0 origin of the display which
 * is the upper left corner.
 * Requests to set pixels outside the range of the display will be ignored.
 *
 * @note
 *
*/
void vfd_gotoxy_text(uint8_t x, uint8_t y)
{
  uint16_t addr;

  addr = VFD_TEXT_HOME +  x + (VFD_TEXT_AREA * VFD_TEXT_VTA * y) +  ((VFD_TEXT_VTA - 1) * y);

  _vfd_set_cursor_address(addr);
}

void vfd_puts_text(const unsigned char *str)
{
  while(*str) vfd_putc_text(*str++);
}
void vfd_puts_text_p(PGM_P str)
{
  while(pgm_read_byte(str)) vfd_putc_text(pgm_read_byte(str++));
}

void vfd_putc_text(uint8_t c)
{
  // Automatic Increment of Cursor address  temporarily On.
  if(VFD_ADDR_AUTO_INC==0) vfd_command(VFD_ADDR_INC_AUTO);

  vfd_command(VFD_DATA_WRITE);
  vfd_data(c);
  
  if(VFD_ADDR_AUTO_INC==0) vfd_command(VFD_ADDR_INC_HOLD);   
}

/*
 * unsigned int to characters in LCD
 *
 * @param value         16bit unsigned int number to be displayed
 * @param radix         2(binary),
 *                     10(decimal),
 *                     16(hexadecimal)
 * @param length       display length (right alignment)
 * @param leadingchar  specify leading char
 *                     '0'  ->  '00099'
 *                     ' '  ->  '   99'
 *                     '$'  ->  '$$$99'
 *                     'x'  ->  'xxx99'
 *
 */
#define NUMBER_OF_DIGITS 16
void vfd_puts_text_utoa(uint16_t value, uint8_t radix, uint8_t length, uint8_t leadingchar)
{
  static uint8_t  index = NUMBER_OF_DIGITS;
  static unsigned char buffer[NUMBER_OF_DIGITS];

  do
  {
    buffer[--index] = '0' + (value % radix);
    if ( buffer[index] > '9') buffer[index] += 'A' - ':';  // continue with A, B,...

    value /= radix;
  } while (value != 0);

  while( index > (NUMBER_OF_DIGITS - length))
  {
  	buffer[--index] = leadingchar;
  }

  if(radix==16) vfd_puts_text_P("0x");
  else
  if(radix==2) vfd_puts_text_P("b");

  do
  {
  	vfd_putc_text(buffer[index++]);
  } while ( index < NUMBER_OF_DIGITS );
}


void vfd_dot(const uint16_t x, const uint8_t y, const uint8_t color)
{
	if( ( x > VFD_X_PIXEL-1) || ( y > VFD_X_PIXEL-1 ) ) return;
	
  uint8_t data,rest;
  rest = y % 8;

  vfd_gotoxy(x, y);

  vfd_command(VFD_DATA_READ);
  data = vfd_read_data();     // read data from display memory

  switch (color)
  {
    case VFD_DOT_SET:
       data |=  (0x80 >> rest);
       break;
    case VFD_DOT_CLR:
 	 	   data &=  (~(0x80 >> rest));
       break;
    case VFD_DOT_XOR:
       data ^=  (0x80 >> rest);
       break;
    default: break;
  }
  vfd_command(VFD_DATA_WRITE);
  vfd_data(data);
  
}
/*
 * Draw line code is based on Bresenham's line algorithm
 * http://en.wikipedia.org/wiki/Bresenham%27s_line_algorithm
 *
 * This routine replaces the Teiele's DrawLine() routine which was
 * much larger and much slower.
 *  -- bperrybap
*/

/*
 * First, define a few macros to make the DrawLine code below read more like
 * the wikipedia example code.
*/

#define _absDiff(x,y) ((x>y) ?  (x-y) : (y-x))
#define _swap(a,b) \
do\
{\
uint16_t t;\
  t=a;\
  a=b;\
  b=t;\
} while(0)
/**
 * Draw a Line
 *
 * @param x a value from 0 to VFD_X_PIXEL-1, indicating start x coordinate
 * @param y a value from 0 to VFD_Y_PIXEL-1, indicating start y coordinate
 * @param height a value from 0 to LCD_RIGHT-y
 * @param color color of line
 * @param style style of line
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * The line drawn will be height+1 pixels.
 *
 * @note
 *
*/
void vfd_line(uint16_t x1, uint8_t y1, uint16_t x2, uint8_t y2, const uint8_t color, const uint8_t style)
{
  uint16_t deltax, x, steep;
  int16_t error, ystep;
  uint8_t deltay,y;

  steep = _absDiff(y1,y2) > _absDiff(x1,x2);

  if ( steep )
  {
    _swap(x1, y1);
    _swap(x2, y2);
  }

  if (x1 > x2)
  {
    _swap(x1, x2);
    _swap(y1, y2);
  }

  deltax = x2 - x1;
  deltay =_absDiff(y2,y1);
  error = deltax / 2;
  y = y1;
  if(y1 < y2) ystep = 1;  else ystep = -1;

  for(x = x1; x <= x2; x++)
  {
    if(steep)
      vfd_dot(y,x, color);
    else
      vfd_dot(x,y, color);

    error = error - deltay;
    if(error < 0)
    {
      y = y + ystep;
      error = error + deltax;
    }
  }
}

/**
 * Draw a Horizontal Line
 *
 * @param x1 a value from 0 to VFD_X_PIXEL-1, indicating start x coordinate
 * @param y1 a value from 0 to VFD_Y_PIXEL-1, indicating start y coordinate
 * @param x2 a value from 0 to VFD_X_PIXEL-1, indicating start x coordinate
 * @param color color of line
 * @param style style of line
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * The line drawn will be width+1 pixels.
 *
 * @note
 *
*/
void vfd_hline(uint16_t x1, const uint8_t y1, uint16_t x2, const uint8_t color, const uint8_t style)
{
   uint16_t x;

   for(x= x1; x<=x2; x += (style+1)) vfd_dot(x,y1,color);

}

/**
 * Draw a Vertical Line
 *
 * @param x1 a value from 0 to VFD_X_PIXEL-1, indicating start x coordinate
 * @param y1 a value from 0 to VFD_Y_PIXEL-1, indicating start y coordinate
 * @param y2 a value from 0 to VFD_Y_PIXEL-1, indicating start y coordinate
 * @param color color of line
 * @param style style of line
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * The line drawn will be height+1 pixels.
 *
 * @note
 *
*/
void vfd_vline(const uint16_t x1, const uint8_t y1, const uint8_t y2, const uint8_t color, const uint8_t style)
{
   uint8_t y;

   for(y = y1; y <= y2; y += (style+1)) vfd_dot(x1, y, color);

   if(style == VFD_LINE_DOTTED) vfd_dot(x1,y2,color);
}

/**
 * Draw a rectangle of given width and height
 *
 * @param x1 X coordinate of upper  left corner of the rectangle
 * @param y1 Y coordinate of upper  left corner of the rectangle
 * @param x2 X coordinate of lower right corner of the rectangle
 * @param y2 Y coordinate of lower right corner of the rectangle
 * @param color color of line
 * @param style style of line
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * Draws a rectangle with the specified width and height.
 * The upper left corner at x,y and the lower right
 * corner at x+width,y+width.
 *
 * The length of the horizontal sides will be width+1 pixels
 * The length of the vertical sides will be height+1 pixels
 *
 * @note
 *
*/
void vfd_rect(const uint16_t x1, const uint8_t y1, const uint16_t x2, const uint8_t y2,
              const uint8_t color, const uint8_t style)
{
  vfd_hline(x1, y1, x2, color, style);  // top
  vfd_hline(x1, y2, x2, color, style);  // bottom
  vfd_vline(x1, y1, y2, color, style);  // left
  vfd_vline(x2, y1, y2, color, style);  // right
}
/**
 * Draw a rectangle with rounder corners
 *
 * @param x1 X coordinate of upper  left corner of the rectangle
 * @param y1 Y coordinate of upper  left corner of the rectangle
 * @param x2 X coordinate of lower right corner of the rectangle
 * @param y2 Y coordinate of lower right corner of the rectangle
 * @param color color of line
 * @param style style of line
 * @param radius
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * Draws a rectangle the same as vfd_rect but with rounded corners.
 * Radius is a value from 1 to half the height or width of the rectangle.
 *
 * @note
 *
*/
void vfd_rect_round(const uint16_t x1, const uint8_t y1, const uint16_t x2, const uint8_t y2,
                    const uint8_t color, const uint8_t style, const uint8_t radius)
{
  int16_t tSwitch;
  uint16_t xoffset;
  uint8_t  yoffset;

  if(radius<1) return;

  xoffset = 0;
  yoffset = radius;
  tSwitch = 3 - 2 * radius;

  while(xoffset <= yoffset)
  {
    vfd_dot(x1+radius - xoffset, y1+radius - yoffset, color);
    vfd_dot(x1+radius - yoffset, y1+radius - xoffset, color);

    vfd_dot(x2-radius + xoffset, y1+radius - yoffset, color);
    vfd_dot(x2-radius + yoffset, y1+radius - xoffset, color);

    vfd_dot(x2-radius + xoffset, y2-radius + yoffset, color);
    vfd_dot(x2-radius + yoffset, y2-radius + xoffset, color);

    vfd_dot(x1+radius - xoffset, y2-radius + yoffset, color);
    vfd_dot(x1+radius - yoffset, y2-radius + xoffset, color);

    if(tSwitch <= 0)
    {
      tSwitch += (4 * xoffset + 6);
    }
    else
    {
      tSwitch += (4 * (xoffset - yoffset) + 10);
      yoffset--;
//      yoffset -= (style+1);
    }
    xoffset++;
//    xoffset += (style+1);
  }

  vfd_hline(x1+ radius+1, y1, x2-radius-1, color, style); // top
  vfd_hline(x1+ radius+1, y2, x2-radius-1, color, style); // bottom
  vfd_vline(x1,y1+radius,     y2-radius,   color, style); // left
  vfd_vline(x2,y1+radius,     y2-radius,   color, style); // right
}
/**
 * Draw a rectangle with fill
 *
 * @param x1 X coordinate of upper  left corner of the rectangle
 * @param y1 Y coordinate of upper  left corner of the rectangle
 * @param x2 X coordinate of lower right corner of the rectangle
 * @param y2 Y coordinate of lower right corner of the rectangle
 * @param color color of line
 * @param style style of line
 * @param radius
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 *
 * Draws a rectangle the same as vfd_rect but with fill area.
 *
 * @note
 *
*/
void vfd_rect_fill(const uint16_t x1, const uint8_t y1, uint16_t x2, const uint8_t y2, const uint8_t color)
{
  uint8_t i;

  for (i=y1; i<=y2; i++) vfd_hline(x1, i, x2, color, VFD_LINE_SOLID);
}

/**
 * Draw a Circle
 *
 * @param xCenter X coordinate of the center of the circle
 * @param yCenter Y coordinate of the center of the circle
 * @param radius radius of circle
 * @param color
 * @param style
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * Draws a circle of the given radius extending out from
 * the center pixel.
 * The circle will fit inside a rectanglular area bounded by
 * x-radius,y-radius and x+radius,y+radius
 *
 * Because the circle is drawn from the center pixel out,
 * the diameter will be 2 * radius +1 pixels.
 *
 * @note
 *
*/
void vfd_circle(const uint16_t xcenter, const uint8_t ycenter, const uint8_t radius, const uint8_t color)
{
  vfd_rect_round(xcenter-radius, ycenter-radius, xcenter+radius, ycenter+radius, color, VFD_LINE_SOLID, radius);
}

/**
 * Draw a cuboid
 *
 * @param x11 X coordinate of upper left point of the first surface
 * @param y11 Y coordinate of upper left point of the first surface
 * @param x21 X coordinate of lower right point of the first surface
 * @param y21 Y coordinate of lower right point of the first surface
 * @param x21 X coordinate of upper left point of the second surface
 * @param y21 Y coordinate of upper left point of the second surface
 * @param x22 X coordinate of lower right point of the second surface
 * @param y22 Y coordinate of lower right point of the second surface
 * @param color color of line
 * @param style style of line
 *
 * color of BLACK,CLEAR or INVERT is a parameter indicating pixel color
 * style of SOLID or DOTTED is a parameter indicating line style
 *
 * @note
 *
*/
void vfd_cube(const uint16_t x11, const uint8_t y11, const uint16_t x12, const uint8_t y12,
              const uint16_t x21, const uint8_t y21, const uint16_t x22, const uint8_t y22,
              const uint8_t color, const uint8_t style)
{
  vfd_rect(x11,y11,x12,y12,color,style);
  vfd_rect(x21,y21,x22,y22,color,style);
  vfd_line(x11,y11,x21,y21,color,style);
  vfd_line(x12,y11,x22,y21,color,style);
  vfd_line(x11,y12,x21,y22,color,style);
  vfd_line(x12,y12,x22,y22,color,style);
}
/**
 * Draw a vfd bitmap image (Internal Memory:Flash)
 *
 * @param bitmap a ponter to the bitmap data
 * @param x the x coordinate of the upper left corner of the bitmap
 * @param y the y coordinate of the upper left corner of the bitmap
 *
 * Draws a bitmap image with the upper left corner at location x,y
 *
 * @note : Bit byte Aligment : LSB to MSB, Top to Bottom !
 */
void vfd_bitmap(uint8_t* bitmap, uint16_t x, uint8_t y)
{
  uint16_t width,w;
  uint16_t height;

  uint8_t data,h;

  width   =  pgm_read_byte_near(bitmap++) << 8;
  width  +=  pgm_read_byte_near(bitmap++);
  if(width > VFD_X_PIXEL-1) width = VFD_X_PIXEL-1;

  height  =  pgm_read_byte_near(bitmap++) << 8;
  height +=  pgm_read_byte_near(bitmap++);
  if(height > VFD_Y_PIXEL-1) height = VFD_Y_PIXEL-1;

  height >>= 3;    // height = height / 8

  vfd_gotoxy(x, y);
  for(h=0; h<height; h++)
  {
    for(w=0; w<width; w++)
    {
      data = pgm_read_byte(bitmap++);

      vfd_gotoxy(x+w, y + (h*8));
      vfd_offsetwrite(data);
   }
  }
}
//
// Low-Level Graphic LCD Functions
//

 /**
 * Cursor Address Set
 *
 * @param addr, a value poistion(13Bit address)
 *
 * The CURSOR address means the address writing/reading address of displaying data in the RAM
 * The Cursor address is divided into the 2 portions as Lower(8Bit) and Upper (5Bit)
 *
 * @note SET LOWER ADDRESS FIRST
 *
*/
static void _vfd_set_cursor_address(uint16_t addr)
{
  vfd_command(VFD_CURSOR_ADDR_L);
  vfd_data(lobyte(addr));

  vfd_command(VFD_CURSOR_ADDR_U);
  vfd_data(hibyte(addr));
}

/*
 * toggle WR Pin to initiate write
 *
 * @return
 *
*/
static void _vfd_write_toggle(void)
{
  // Write Signal Toggle
  VFD_CTRL_PORT &= ~(_BV(VFD_WR_PIN));
  VFD_CTRL_PORT |=   _BV(VFD_WR_PIN);

  // Since data excution time of the module is 2uSec
  // There is not provided Busy signal therefore more than 2uSec.
  // Refer. Datasheet 13. Interfacing, 14. Timing Chart
  _delay_us(2);
}
/**
 * EOF
*/


