#ifndef VFD_H_
#define VFD_H_
/*
 * $Id: vfd.h 2013-09-16 PM 7:14:36 ssk  $
 *
 * This file is part of  Noritake itron GU256X64-901A 256x64 Dot Matrix VFD Tutorial.
 * Designed for The Noritake high speed 8 bit parallel interface based VFDs on Atmels AVR MCUs
 *
 * Copyright (C) 2013  ssk.
 *
 * Originally Adapted from
 *    Most Functions   : GLCD Version 3 (Beta) Jun 9 2010 <http://arduino.cc/forum/index.php/topic,22208.0.html>
 *    Font Function&Data: Martin Thomas, Kaiserslautern, Germany <eversmith@heizung-thomas.de>
 *                       avr-gcc 3.4.3/avr-libc 1.2.3 (WinAVR 2/2005)
 *    Cube Drawing     : Osama's Lab KS0108 GLCD library <https://sites.google.com/site/osamaslab>
 *    Line Drawing     : http://en.wikipedia.org/wiki/Bresenham's_line_algorithm
 *    Circle Drawing   : http://en.wikipedia.org/wiki/Midpoint_circle_algorithm
 *
 * Changed & added additional constants for gcc version 4.3.3 (WinAVR 20100110), improved speed and optimized code
 * You need to be selective about the information you receive instead of accepting it unconditionally.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
*/
#include <avr/pgmspace.h>

// This VFD has 15Bit Address Structure (0x00 ~ 0x1ff)
// Lower(8Bit) and Upper (5Bit)
#define	lobyte(x) (uint8_t) ((x) & 0x00ff)
//#define	hibyte(x) (uint8_t)(((x) & 0xff00) >> 8)
#define	hibyte(x) (uint8_t)(((x) & 0x1f00) >> 8)

// data port                
#define VFD_DATA_PORT         PORTB
#define VFD_DATA_INPUT        PINB
#define VFD_DATA_DDR          DDRB
                              
// control port               
#define VFD_CTRL_PORT         PORTA
#define VFD_CTRL_DDR          DDRA
                              
// control signals            
#define VFD_WR_PIN            PA5   // Write Signal signal. (Input)
#define VFD_CD_PIN            PA6   // Command/Data Select Signal (Input), CD= 1':Command, CD='0':Data
#define VFD_RD_PIN            PA7   // Read Signall signal. (Input)


// VFD Controller Board Configurations
#define VFD_SRAM_SZ           8192  // embedded display memory size (8Kbyte:8192 x 8 bits)

// Cursor address behavior
#define VFD_ADDR_AUTO_INC     0     // Cursor address Increment behavior 0:Hold,1:Autoincrement)

                            
// Display properties       
// Graphic Area, Screen #1
#define VFD_X_PIXEL		        256   // The number of Width  Dots 
#define VFD_Y_PIXEL	          64    // The number of Height Dots 
#define VFD_Y_PAGES	          8     // The number of Page (8page x 8bit = 64 pixels)
#define VFD_X_CENTER         (VFD_X_PIXEL / 2)    // the column  center of pixel 
#define VFD_Y_CENTER         (VFD_Y_PIXEL / 2)    // the row     center of pixel 

// Text Area, Screen #2(Power On Default)
#define VFD_TEXT_FONT_WIDTH   6     // 6:6x8
#define VFD_TEXT_FONT_HEIGHT  8     // 8:6x8
#define VFD_X_TEXT           (VFD_X_PIXEL / VFD_TEXT_FONT_WIDTH)   // The number of Char per Column
#define VFD_Y_TEXT           (VFD_X_PIXEL / VFD_TEXT_FONT_HEIGHT)  // The number of Char per Line
#define VFD_X_TEXT_CENTER   ((VFD_X_PIXEL / 8) / 2)  // the column center of text
#define VFD_Y_TEXT_CENTER    (VFD_Y_PAGES / 2)       // the row    center of Text 
#define VFD_TEXT_VTA          3     // Virtual Screen Adddress (VFD_X_TEXT * VFD_TEXT_VTA)
                                    // Refer. Datasheet 12.2 Character Display map
// indicate how many bytes are used for one complete row (number of Column)
#define VFD_GRAPHIC_AREA     (VFD_X_PIXEL)
#define VFD_TEXT_AREA        (VFD_X_TEXT)
                           
// indicate how many bytes are used for Graphic area and Text area
#define VFD_GRAPHIC_SIZE     (VFD_GRAPHIC_AREA * VFD_Y_PAGES)
#define VFD_TEXT_SIZE        (VFD_TEXT_AREA    * VFD_TEXT_FONT_HEIGHT)
                           
// They are used to set the beginning of the text area and graphic area in the RAM
#define VFD_GRAPHIC_HOME      0
#define VFD_TEXT_HOME        (VFD_GRAPHIC_HOME + VFD_GRAPHIC_SIZE)

// command function equates for Noritake GU256X64-901A VFD Controller
// * Marks are Power On Default
#define VFD_SCREEN_OFF		    0x00   // *Screen #1,#2 Off  
#define VFD_SCREEN1_ON 	      0x01   //  Screen #1 On, #2 Off (Graphic)
#define VFD_SCREEN2_ON 	      0x02   //  Screen #2 On, #1 Off (Text, Power On Default)
#define VFD_SCREEN_ON			    0x03   //  Screen #1,#2 On

#define VFD_ADDR_INC_AUTO     0x04   //  Automatic Increment of Cursor address
#define VFD_ADDR_INC_HOLD	    0x05   //  Hold Increment of Cursor address
                              
#define VFD_SCREEN_OR			    0x10   // *OR  Display Screen #1,#2
#define VFD_SCREEN_XOR  	    0x11   //  XOR Display Screen #1,#2
#define VFD_SCREEN_AND  	    0x12   //  AND Display Screen #1,#2
                              
#define VFD_DISPLAY_ON   	    0x14   // *Display On
#define VFD_DISPLAY_OFF       0x15   //  Display Off
                              
#define VFD_CURSOR_INC_ON     0x04   // *Cursor is automatically increment
#define VFD_CURSOR_INC_OFF    0x05   //  Cursor is Not automatically increment
                              
#define VFD_SCREEN2_CHAR      0x06   // *Screen #2 is selected to Character Display
#define VFD_SCREEN2_GRAP      0x07   //  Screen #2 is selected to Graphic Display
                              
#define VFD_DATA_WRITE        0x08   // Set Data(pattern or character) Write mode
#define VFD_DATA_READ         0x09   // Set Data(pattern or character) Read mode
                              
#define VFD_START_ADDR_L1     0x0a   // Lower byte of Screen#1(Graphic) address
#define VFD_START_ADDR_U1     0x0b   // Upper byte of Screen#1(Graphic) address
#define VFD_START_ADDR_L2     0x0c   // Lower byte of Screen#2(Text) address
#define VFD_START_ADDR_U2     0x0d   // Upper byte of Screen#2(Text) address
                              
// The CURSOR address means writing/reading address of displaying data in the RAM                         
#define VFD_CURSOR_ADDR_L     0x0e   // Set Lower byte of cursor address
#define VFD_CURSOR_ADDR_U     0x0f   // Set Upper byte of cursor address
                              
#define VFD_BRIGHTNESS_100    0x18   // *Set brightness 100.0%
#define VFD_BRIGHTNESS_87     0x19   //  Set brightness 87.5%
#define VFD_BRIGHTNESS_75     0x1a   //  Set brightness 75.0%
#define VFD_BRIGHTNESS_62     0x1b   //  Set brightness 62.5%
                             
/* VFD Color modes */        
#define VFD_DOT_CLR           0
#define VFD_DOT_SET           1
#define VFD_DOT_XOR           2
                              
/* VFD Line Style */          
#define VFD_LINE_SOLID        0
#define VFD_LINE_DOTTED       1

#ifndef GLCD_ITYPES             
#define GLCD_ITYPES             
typedef struct coordinates      
{                               
	uint16_t x;        // 0..255               
	uint8_t y;         // 0..63 (LOOK! datatype uint8_t) 
	
//uint8_t text_x;    // 0..41               
//uint8_t text_y;    // 0..7
} coordinates;                  
#endif                          
extern coordinates	_vfd_coord;

// function prototype
extern void vfd_init(void);
extern void vfd_command(const uint8_t command);
extern void vfd_data(const uint8_t data);
extern void vfd_offsetwrite(uint8_t data);
extern void vfd_clear(uint8_t fillchar);
extern void vfd_clear_text(uint8_t fillchar);
extern void vfd_gotoxy(uint16_t x,  uint8_t y);
extern void vfd_gotoxy_text(uint8_t x, uint8_t y);
extern void vfd_putc_text(uint8_t c);                // Text mode put char  
extern void vfd_puts_text(const unsigned char *str); // Text mode put string
//extern void vfd_puts_text_p(const char *progmem_s);// Text mode put string PROGMEM 
extern void vfd_puts_text_p(PGM_P str);              // Text mode put string PROGMEM 
extern void vfd_puts_text_utoa(uint16_t value, uint8_t radix, uint8_t length, uint8_t leadingchar);
#define vfd_puts_text_P(__s) vfd_puts_text_p(PSTR(__s))

extern void vfd_dot(const uint16_t x, const uint8_t y, const uint8_t color);  
extern void vfd_line(uint16_t x1, uint8_t y1, uint16_t x2, uint8_t y2, const uint8_t color, const uint8_t style);
extern void vfd_hline(uint16_t x1, const uint8_t y1, uint16_t x2, const uint8_t color, const uint8_t style);
extern void vfd_vline(const uint16_t x1, const uint8_t y1, const uint8_t y2, const uint8_t color, const uint8_t style);
extern void vfd_rect(const uint16_t x1, const uint8_t y1, const uint16_t x2, const uint8_t y2,
                     const uint8_t color, const uint8_t style);
extern void vfd_rect_round(const uint16_t x1, const uint8_t y1, const uint16_t x2, const uint8_t y2, \
                           const uint8_t color, const uint8_t style, const uint8_t radius);
extern void vfd_rect_fill(const uint16_t x1, const uint8_t y1, uint16_t x2, const uint8_t y2, const uint8_t color);
extern void vfd_circle(const uint16_t xcenter, const uint8_t ycenter, const uint8_t radius, const uint8_t color);
extern void vfd_cube(const uint16_t x11, const uint8_t y11, const uint16_t x12, const uint8_t y12,
                     const uint16_t x21, const uint8_t y21, const uint16_t x22, const uint8_t y22,
                     const uint8_t color, const uint8_t style);
extern void vfd_bitmap(uint8_t* bitmap, uint16_t x, uint8_t y);
extern uint8_t vfd_read_data(void);

#endif

